package pkg.sample.imageprocessing;

import android.app.Activity;
import android.content.Context;
import android.content.pm.ActivityInfo;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Matrix;
import android.graphics.Paint;
import android.graphics.PixelFormat;
import android.graphics.PointF;
import android.media.FaceDetector;
import android.media.FaceDetector.Face;
import android.opengl.GLSurfaceView;
import android.os.Bundle;
import android.util.Log;
import android.view.KeyEvent;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.view.WindowManager;

public class ImageProcessing extends Activity {
	FrameRateChecker mFPS = new FrameRateChecker(10); // t[[gmFp

	CameraCallback mCameraCallback = null; // JpNX
	GraphicsView mGraphicsView = null; // 摜`pNX
	DrawSurfaceView mDrawSurfaceView = null;

	OverlayGLSurfaceView mGLSurfaceView = null;

	ColorConvert mColorConvert = new ColorConvert();
	ColorConvert.YUV420toRGB8888 mYUV2RGB = null; // YUV420->ARGB8888 FϊNX

	int[] mIntImage = null; // ARGB8888 int^摜z
	byte[] mYBuffer = null;
	int mWidth = 240, mHeight = 160; // vr[TCY
	int mYStep = 2, mCrStep = 3, mCbStep = 3; // YUV̊Ԉ
	int mFrameCount = 0; // 񐔂̃JEg

	float mSizeWidth = 1, mSizeHeight = 1;

	boolean mNativeFlag = true;// JNIgǂ

	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		this.setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE); // ʂ̌ŌŒ
		this.getWindow().addFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN); // Sʕ\
		this.getWindow().addFlags(
				WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON); // X[v
		this.requestWindowFeature(Window.FEATURE_NO_TITLE); // ^Cg̔\


		final ViewGroup.LayoutParams layoutParams = new ViewGroup.LayoutParams(
				ViewGroup.LayoutParams.FILL_PARENT,
				ViewGroup.LayoutParams.FILL_PARENT);

		mCameraCallback = new CameraCallback(this);
		mCameraCallback.setClass(this);

		// View
		mGraphicsView = new GraphicsView(this);
		setContentView(mCameraCallback, layoutParams);
		addContentView(mGraphicsView, layoutParams);

		// SurfaceView
		/*
		 * setContentView(R.layout.main); mDrawSurfaceView
		 * =(DrawSurfaceView)findViewById(R.id.DrawSurfaceClass);
		 * addContentView(mCameraCallback, layoutParams);
		 */

		// GL
		/*
		 * setContentView(R.layout.main); mGLSurfaceView
		 * =(OverlayGLSurfaceView)findViewById(R.id.OpenGLClass);
		 * addContentView(mCameraCallback, layoutParams);
		 */

		//\
		addContentView(new StatusView(this), layoutParams);

	}

	// \pView
	public class StatusView extends View {
		Paint paint;
		String mStatusStr[] = { "", "", "", "", "" };

		public StatusView(Context context) {
			super(context);
			paint = new Paint();
		}
		


		@Override
		protected void onDraw(Canvas canvas) {
			if (mFrameCount % 5 == 0) {
				if (mNativeFlag)
					mStatusStr[0] = mFPS.getString()
							+ "fps ImageProcessing(NDK)";
				else
					mStatusStr[0] = mFPS.getString() + "fps ImageProcessing";
				if (mCameraCallback != null)
					mStatusStr[1] = mCameraCallback.mFPS.getString()
							+ "fps CameraPreview";

				if (mGLSurfaceView != null)
					mStatusStr[2] = mGLSurfaceView.mFPS.getString()
							+ "fps View";
				else if (mGraphicsView != null)
					mStatusStr[2] = mGraphicsView.mFPS.getString() + "fps View";

				mStatusStr[3] = "color:" + mYUV2RGB.getColorDepth()
						+ " table size:"
						+ (mYUV2RGB.getColorDepth() * 4 / 1024) + "KB";
				mStatusStr[4] = "width:" + mWidth + " height:" + mHeight;
			}
			paint.setColor(Color.rgb(255, 255, 255));
			canvas.drawText(mStatusStr[0], 10, 10, paint);
			canvas.drawText(mStatusStr[1], 10, 10 + 12, paint);
			canvas.drawText(mStatusStr[2], 10, 10 + 12 * 2, paint);
			canvas.drawText(mStatusStr[3], 256, 10, paint);
			canvas.drawText(mStatusStr[4], 256, 10 + 12, paint);
		}
	}

	// vr[`pView
	private class GraphicsView extends View {
		private FrameRateChecker mFPS = new FrameRateChecker(10);
		Paint paint;

		public GraphicsView(Context context) {
			super(context);
			paint = new Paint();
		}

		@Override
		protected void onDraw(Canvas canvas) {
			this.mFPS.measurement();
			canvas.drawRGB(128, 128, 128);
			canvas.scale(mSizeWidth, mSizeHeight); // 摜ʃTCYŕ\
			canvas.drawBitmap(mIntImage, 0, mWidth, 0, 0, mWidth, mHeight,
					false, paint); // int^z𒼐ڕ`
		}

	}
	
	// L[
	@Override
	public boolean dispatchKeyEvent(KeyEvent event) {
		if (event.getAction() == KeyEvent.ACTION_UP) { // L[ꂽ
			switch (event.getKeyCode()) {
			case KeyEvent.KEYCODE_DPAD_CENTER: // L[
				mNativeFlag = !mNativeFlag;//JavaNDK؂ւ
				return true;
			default:
			}
		}
		return super.dispatchKeyEvent(event);
	}
	
	// CameraCallbackĂяo鏉p\bh
	public void initImageData(int cam_w, int cam_h, int scr_w, int scr_h) {
		mIntImage = new int[cam_w * cam_h];
		mYBuffer = new byte[cam_w * cam_h];
		mWidth = cam_w;
		mHeight = cam_h;

		mSizeWidth = (float) scr_w / cam_w;
		mSizeHeight = (float) scr_h / cam_h;
		
		initTable(cam_w, cam_h, mYStep, mCrStep, mCbStep);

		if (mGLSurfaceView != null)
			mGLSurfaceView.setTextureSize(cam_w);

		mYUV2RGB = mColorConvert.newYUV420toRGB8888(cam_w, cam_h, mYStep,
				mCrStep, mCbStep);
	}

	// CameraCallbackĂяoJp\bh
	void cameraLoop(byte yuv_data[]) {
		mFrameCount++;
		mFPS.measurement();//N[[ǧv(XbhŏĂȂ̂onPreviewFrameƂ)

		if (!mNativeFlag) {// Javaŏ
			// ImageFilter3x3(yuv_data,mFilter3x3_sharp1);//tB^[
			// mYUV2RGB.getQuarter(yuv_data, mIntImage,mFrameCount);//RGBϊ܂т
			mYUV2RGB.getFast(yuv_data, mIntImage);
			return;
		}
		
		//NDKŏ̏ꍇ
		if (mGLSurfaceView != null) {// OpenGLŕ`悷ꍇ
			convertYUV2RGB565Tex(yuv_data, mGLSurfaceView.mUShortData, 0);// YUV420->RGB565
			mGLSurfaceView.draw(0, 0);
		} else {
			//Viewŕ`悷ꍇ
			convertYUV2RGB8888(yuv_data, mIntImage, 0);//YUV420->ARGB8888

			if (mDrawSurfaceView != null) {// SurfaceViewŕ`悷ꍇ
				Canvas canvas = mDrawSurfaceView.LockCanvas();
				if (canvas != null) {
					mGraphicsView.draw(canvas);
					mDrawSurfaceView.UnlockCanvas(canvas);
				}
			}
		}

	}

	int mFilter3x3_sharp1[] = { 0, -1, 0, -1, 5, -1, 0, -1, 0, 1 };
	int mFilter3x3_sharp2[] = { -1, -1, -1, -1, 9, -1, -1, -1, -1, 1 };
	int mFilter3x3_smooth1[] = { 0, 1, 0, 1, 1, 1, 0, 1, 0, 5 };
	int mFilter3x3_smooth2[] = { 1, 1, 1, 1, 1, 1, 1, 1, 1, 9 };
	int mFilter3x3_edge[] = { -1, -1, -1, -1, 8, -1, -1, -1, -1, 1 };

	void ImageFilter3x3(byte[] aSrc, int[] filter) {
		int pixPtr;
		for (int j = 1; j < mHeight - 1; j++) {
			pixPtr = j * mWidth + 1;
			for (int i = 1; i < mWidth - 1; i++) {
				int v = ((aSrc[pixPtr - 1 - mWidth] & 0xff) * filter[0]
						+ (aSrc[pixPtr - mWidth] & 0xff) * filter[1]
						+ (aSrc[pixPtr + 1 - mWidth] & 0xff) * filter[2]
						+ (aSrc[pixPtr - 1] & 0xff) * filter[3]
						+ (aSrc[pixPtr] & 0xff) * filter[4]
						+ (aSrc[pixPtr + 1] & 0xff) * filter[5]
						+ (aSrc[pixPtr - 1 + mWidth] & 0xff) * filter[6]
						+ (aSrc[pixPtr + mWidth] & 0xff) * filter[7] + (aSrc[pixPtr
						+ 1 + mWidth] & 0xff)
						* filter[8])
						/ filter[9];
				if (v > 255)
					v = 255;
				else if (v < 0)
					v = 0;
				mYBuffer[pixPtr] = (byte) v;
				pixPtr++;
			}
		}
		System.arraycopy(mYBuffer, 0, aSrc, 0, mWidth * mHeight);
	}

	static {
		System.loadLibrary("picture");
	}

	private native void initTable(int widht, int height, int aY, int aCr,
			int aCb);

	private native int convertYUV2RGB8888(byte[] src, int[] dst, int flag);

	private native int convertYUV2RGB8888Quarter(byte[] src, int[] dst,
			int flag, int count);

	private native int convertYUV2RGB565Tex(byte[] src, byte[] tex, int flag);
	

}